package move;

import java.util.*;
import java.awt.*;
import java.awt.image.*;
import javax.imageio.*;
import java.io.*;

// class handling images
public class ImageFile {
	private String file;				// file representing image
	private Frame f;					// frame creating buffered image
	private BufferedImage img=null;		// the image data

	// creates image from file using frame f
	public ImageFile(String file, Frame f) {
		this.file=file;
		this.f=f;
		try { img = ImageIO.read(new File(file)); }
		catch(Exception e) { System.out.println(e); }
		//Image i = Toolbox.getImage(file,f,false);
		if(img==null) return;
		while(img.getWidth(f)==-1);
		while(img.getHeight(f)==-1);
		//img= new BufferedImage(i.getWidth(f),i.getHeight(f),BufferedImage.TYPE_INT_RGB);
		Graphics g=img.getGraphics();
		while(!g.drawImage(img,0,0,f)) Toolbox.sleep(10);
	}
	// returns image
	public BufferedImage getImage() { return img; }
	// returns part of image as image zoomed by factor zoom
	public Image getPart(int x, int y, int w, int h, int zoom) {
		if(img==null) return null;
		Image r = f.createImage(w*zoom,h*zoom);
		Graphics g = r.getGraphics();
		for(int i=0; i<w; i++)
			for(int j=0; j<h; j++) {
				g.setColor(new Color(img.getRGB(x+i,y+j)));
				g.fillRect(i*zoom,j*zoom,zoom,zoom);
		}
		return r;
	}
	// finds markers result for image using markers marks of image
	public Marker detect(ImageFile image, MarkerFile marks, MarkerFile result) {
		Marker n = null;
		marks.load();
		result.load();
		BufferedImage bi = image.getImage();
		for(Enumeration e = marks.elements(); e.hasMoreElements();) {
			Marker m = (Marker)e.nextElement();
			if(!result.exists(m.name)) {
				n = find(m.name,bi,m.x,m.y);
				if(n!=null) result.put(n);
				else { result.store(); return m; }
			}
		}
		result.store();
		return null;
	}
	// search routine
	private Marker find(String name, BufferedImage bim, int x, int y) {
		int	min=0xffffff,sum,xx=0,yy=0;
		for(int dx=-7; dx<8; dx++)
			for(int dy=-7; dy<8; dy++) {
				sum = diff(bim,x,y,dx,dy);
				if(sum<min && (img.getRGB(x+dx,y+dy)&0xff)>(bim.getRGB(x,y)&0xff-20)) {
					min=sum; xx=dx; yy=dy;
				}
		}
		if((img.getRGB(x+xx,y+yy)&0xff)<60) return null;
		else return new Marker(name,x+xx,y+yy,255);
	}
	// calculates value representing difference between two images
	private int diff(BufferedImage bim, int x, int y, int dx, int dy) {
		int sum=0,vo,vn,ax,ay;
		for(int i=-7; i<8; i++)
			for(int j=-7; j<8; j++) {
				vo = bim.getRGB(x+i,y+j)&0xff;
				vn = img.getRGB(x+dx+i,y+dy+j)&0xff;
				ax = 8-Math.abs(i);
				ay = 8-Math.abs(j);
				sum+=(ax*ay*Math.abs(vo-vn));
		}
		return sum;
	}
}
