package move;

import java.util.*;
import java.awt.*;
import java.awt.image.*;

// class handling images
public class ImageFile {
	private String file;				// file representing image
	private Frame f;					// frame creating buffered image
	private BufferedImage img=null;		// the image data
	private int[][] ar;					// array of differences for autodetection

	// creates image from file using frame f
	public ImageFile(String file, Frame f) {
		this.file=file;
		this.f=f;
		Image i = Toolbox.getImage(file,f,false);
		if(i==null) return;
		while(i.getWidth(f)==-1);
		while(i.getHeight(f)==-1);
		img= new BufferedImage(i.getWidth(f),i.getHeight(f),BufferedImage.TYPE_INT_RGB);
		Graphics g=img.getGraphics();
		while(!g.drawImage(i,0,0,f)) Toolbox.sleep(10);
	}
	// returns image
	public BufferedImage getImage() { return img; }
	// returns part of image as image zoomed by factor zoom
	public Image getPart(int x, int y, int w, int h, int zoom) {
		if(img==null) return null;
		Image r = f.createImage(w*zoom,h*zoom);
		Graphics g = r.getGraphics();
		for(int i=0; i<w; i++)
			for(int j=0; j<h; j++) {
				g.setColor(new Color(img.getRGB(x+i,y+j)));
				g.fillRect(i*zoom,j*zoom,zoom,zoom);
		}
		return r;
	}
	// finds markers result for image using markers marks of image
	public Marker detect(ImageFile image, MarkerFile marks, MarkerFile result) {
		Marker n = null;
		marks.load();
		result.load();
		BufferedImage bi = image.getImage();
		for(Enumeration e = marks.elements(); e.hasMoreElements();) {
			Marker m = (Marker)e.nextElement();
			if(!result.exists(m.name)) {
				n = find(m.name,bi,m.x,m.y);
				if(n!=null) result.put(n);
				else { result.store(); return m; }
			}
		}
		result.store();
		return null;
	}
	// search routine
	private Marker find(String name, BufferedImage bim, int x, int y) {
		int	min=0xffffff,sum,xx=0,yy=0;
		for(int dx=-3; dx<4; dx++)
			for(int dy=-3; dy<4; dy++) {
				sum = diff(bim,x,y,dx,dy);
				if(sum<min) { min=sum; xx=dx; yy=dy; }
		}
		if(min>0xffffff) return null;
		else return new Marker(name,x+xx,y+yy,255);
	}
	private int diff(BufferedImage bim, int x, int y, int dx, int dy) {
		int sum=0,vo,vn,ax,ay,df;
		set(bim,x,y);
		for(int i=-5; i<6; i++)
			for(int j=-5; j<6; j++) {
				vo = bim.getRGB(x+i,y+j)&0xff;
				vn = img.getRGB(x+dx+i,y+dy+j)&0xff;
				ax = 6-Math.abs(i);
				ay = 6-Math.abs(j);
				df = Math.abs(vo-vn);
				sum+=(ax*ay*df*df*ar[i+5][j+5]);
		}
		return sum-36*(img.getRGB(x+dx,y+dy)&0xff);
	}
	private void set(BufferedImage bim, int x, int y) {
		ar = new int[11][11];
		for(int i=-5; i<6; i++)
			for(int j=-5; j<6; j++) {
				int v = 0;
				v += Math.abs(bim.getRGB(x+i-2,y+i)&0xff-bim.getRGB(x+i+2,y+i)&0xff);
				v += Math.abs(bim.getRGB(x+i,y+i-2)&0xff-bim.getRGB(x+i,y+i+2)&0xff);
				ar[i+5][j+5]=v;
		}
	}
}
