package move;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

public class UserInterface extends FullScreen implements KeyListener,MouseListener,MouseMotionListener {
	private static final int NOOP = 0;								// no operation
	private static final int GOTO = 1;								// goto index
	private static final int SET1 = 2;								// set marker select in image
	private static final int SET2 = 3;								// set marker select in detail
	private static final int SET3 = 4;								// set or new marker type name
	private static final int DEL1 = 5;								// delete marker name
	private static final int DEL2 = 6;								// delete marker to
	private static final int IPOL = 7;								// interpolation
	private static final int AUTO = 8;								// auto detection
	private static final int QUIT = 9;								// quit

	private static final int sw = 1024,	sh = 768;					// screen  dimension
	private static final int px =  352,	py =  32;					// player  position
	private static final int pw =  640,	ph = 512;					// player  dimension
	private static final int dx =   32,	dy = 144;					// detail  position
	private static final int dw =  256,	dh = 256;					// detail  dimension
	private static final int ex =   32,	ey = 464;					// extra   position
	private static final int ew =  256,	eh = 256;					// extra   dimension
	private static final int cx =  352,	cy = 608;					// console position
	private static final int cw =  640,	ch = 112;					// console dimension

	private Graphics g;												// Graphics of Frame
	private ImageFile img=null;										// current ImageFile
	private MarkerFile mrk=null;									// current MarkerFile
	private int index=0;											// index of file
	private int mx,my;												// position of (new) marker

	private int mode=NOOP;											// command-key
	private String read,store;										// read String for keys

	private String path;											// path to files

	public UserInterface(String path, String name, int bpp) throws Exception {
		super(sw,sh,bpp,false);
		this.path=path+Toolbox.FileSep+name;
		g = getGraphics();
		while(!g.drawImage(Toolbox.getImage("move"+Toolbox.FileSep+"move.gif",this,true),0,0,sw,sh,this)) Toolbox.sleep(10);
		g.setFont(new Font("Courier New",Font.PLAIN,12));
		g.setColor(new Color(128,0,0));
		int i=45;
		g.drawString("g.....",ex+2,ey+i); i+=16;
		g.drawString("+|-...",ex+2,ey+i); i+=16;
		g.drawString("m.....",ex+2,ey+i); i+=32;
		g.drawString("d.....",ex+2,ey+i); i+=48;
		g.drawString("i.....",ex+2,ey+i); i+=32;
		g.drawString("a.....",ex+2,ey+i); i+=16;
		g.drawString("c.....",ex+2,ey+i); i+=48;
		g.drawString("q.....",ex+2,ey+i); i =13;
		g.setFont(new Font(null,Font.PLAIN,12));
		g.setColor(Color.black);
		g.drawString("Keyboard Control:",ex+48,ey+i); i+=32;
		g.drawString("goto image with certain index",ex+48,ey+i); i+=16;
		g.drawString("step to next|last image",ex+48,ey+i); i+=16;
		g.drawString("set position of any marker",ex+48,ey+i); i+=16;
		g.drawString("or create a new marker",ex+48,ey+i); i+=16;
		g.drawString("delete marker",ex+48,ey+i); i+=16;
		g.drawString("if typed index is below current index",ex+48,ey+i); i+=16;
		g.drawString("only the current marker will be deleted",ex+48,ey+i); i+=16;
		g.drawString("interpolation",ex+48,ey+i); i+=16;
		g.drawString("finds markers between two markers",ex+48,ey+i); i+=16;
		g.drawString("autodetection (s...for stop)",ex+48,ey+i); i+=16;
		g.drawString("check markers",ex+48,ey+i); i+=16;
		g.drawString("tests if all markers are defined",ex+48,ey+i); i+=16;
		g.drawString("or if there are any unused markers",ex+48,ey+i); i+=16;
		g.drawString("quit",ex+48,ey+i);
		addKeyListener(this);
        addMouseListener(this);
        addMouseMotionListener(this);
        Console.println("(C)2oo4 M|chaeL bL|eM");
        Console.println("");
        paintImage();
	}
	public void start() {
        while(mode!=QUIT) { paintConsole(); if(mode==AUTO) detectMarkers(); Toolbox.sleep(100); }
	}
	// creates filename with path for index with suffix
	private String file(int index, String suffix) { return path+Toolbox.getString(index,4)+"."+suffix; }

	// paints Image
	private void paintImage() {
		img = new ImageFile(file(index,"jpg"),this);
		Image i = img.getImage();
		if(i==null) Console.println("Image for index "+index+" not found.");
		else {
			while(!g.drawImage(i,px,py,pw,ph,this)) Toolbox.sleep(10);
			paintMarker();
			Console.println("Image "+index+" painted."); }
	}
	// paints markers for Image
	private void paintMarker() {
		mrk = new MarkerFile(file(index,"mrk"));
		mrk.load();
		for (Enumeration e = mrk.elements(); e.hasMoreElements();) {
			Marker m = (Marker)e.nextElement();
			// g.setColor(new Color(255-m.q,m.q,0)); // color depends on quality
			g.setColor(new Color(0,128,255));
			g.drawOval(px+m.x/2-3,py+m.y/2-3,7,7);
			g.drawString(m.name,px+m.x/2+5,py+m.y/2+5);
		}
	}
	// paints Details of Image
	private void paintDetail(int x, int y) {
		if(img==null) return;
		x=(x-px)*2;
		y=(y-py)*2;
		if(x<=16) x=16; else if(x>(pw*2-16)) x=pw*2-16;
		if(y<=16) y=16; else if(y>(ph*2-16)) y=ph*2-16;
		while(!g.drawImage(img.getPart(x-16,y-16,32,32,8),dx,dy,this)) Toolbox.sleep(10);
		mx=x-16; my=y-16;
	}
	// paints console of Strings
	private void paintConsole() {
		String[] s = Console.getMessages();
		g.setColor(Color.white);
		g.fillRect(cx,cy,cw,ch);
		g.setColor(Color.black);
		if(s==null) return;
		for(int i=0; i<s.length; i++) g.drawString(s[i],cx+2,cy+13+i*16);
	}
	// checks if markers after current index are available or new
	private void checkMarkers() {
		Console.println("Checking markers:");
		int i=index;
		while(Toolbox.exists(file(i,"jpg"))) {
			if(!Toolbox.exists(file(i,"mrk"))) { Console.println("No markers for index "+i+"."); return; }
			MarkerFile mf = new MarkerFile(file(i,"mrk"));
			mf.load();
			int amt=mrk.size();
			for(Enumeration e = mf.elements(); e.hasMoreElements();) {
				Marker m = (Marker)e.nextElement();
				if(mrk.exists(m.name)) amt--;
				else { Console.println("Marker "+m.name+" found at index "+i+"."); return; }
			}
			if(amt>0) { Console.println(""+amt+" missing marker(s) at index "+i+"."); return; }
			else Console.println("Index "+(i++)+" checked.");
		}
		Console.println("All markers found.");
	}
	// finds missing markers by autodetection
	private void detectMarkers() {
		if(mrk.size()==0) { Console.println("There are no defined markers."); mode=NOOP; return; }
		if(Toolbox.exists(file(++index,"jpg"))) {
			ImageFile ti = new ImageFile(file(index,"jpg"),this);
			MarkerFile tm = new MarkerFile(file(index,"mrk"));
			Marker m = ti.detect(img,mrk,tm);
			ti=null;
			tm=null;
			System.gc();
			if(m!=null) { Console.println("Autodetection stopped: Marker "+m.name+" not found."); mode=NOOP; };
			paintImage();
			Toolbox.sleep(100);
		}
		else { Console.println("No more images available."); mode=NOOP; }
	}
	// finds missing markers by interpolation
	private void interpolate(String name) {
		Console.println("Interpolating marker "+name+":");
		if(!mrk.exists(name)) { Console.println("Marker "+name+" not found."); return; }
		int i=index,end=0,xs=mrk.get(name).x,ys=mrk.get(name).y,xe=0,ye=0;
		while(end<=0 && Toolbox.exists(file(++i,"jpg"))) {
			MarkerFile mf = new MarkerFile(file(i,"mrk"));
			mf.load();
			if(mf.exists(name)) {
				if(i==index+1) end=-1;
				if(end==0) {
					xe=mf.get(name).x;
					ye=mf.get(name).y;
					end=i;
				}
			} else if(end==-1) {
				Console.println("Marker "+name+" is defined until index "+(i-1)+"."); return;
			}
		}
		if(end==0) { Console.println("Marker "+name+" is only defined once."); return; }
		double xf = ((double)(xe-xs))/((double)(end-index));
		double yf = ((double)(ye-ys))/((double)(end-index));
		for(i=index+1; i<end; i++) {
			MarkerFile mf = new MarkerFile(file(i,"mrk"));
			mf.load();
			mf.put(new Marker(name,(int)(xs+(i-index)*xf),(int)(ys+(i-index)*yf),255));
			mf.store();
		}
		Console.println("Markers between "+index+" and "+end+" interpolated.");
	}
    // handles keyboard events
    public void keyPressed(KeyEvent e) {}
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {
		char k=e.getKeyChar();
		switch(mode) {
			case AUTO:
				if(k=='s' || k=='S') { Console.println("Autodetection stopped."); mode=NOOP; }
				break;
			case GOTO:
				if(k!='\n') read+=k;
				else {
					try { index=Integer.parseInt(read); }
					catch(Exception x) { Console.println(x.getMessage()+" is no valid index."); }
					paintImage(); mode=NOOP; }
				break;
			case SET3:
				if(k!='\n') read+=k;
				else { mrk.put(new Marker(read,mx,my,255)); mrk.store(); paintImage(); mode=NOOP; }
				break;
			case DEL1:
				if(k!='\n') read+=k;
				else {
					Console.println("Type last index of marker "+read+" to be deleted:");
					store=read; read=""; mode=DEL2; }
				break;
			case DEL2:
				if(k!='\n') read+=k;
				else {
					if(mrk.exists(store)) {
						try {
							int val=Integer.parseInt(read);
							if(val>index) {
								for(int i=index+1; i<=val; i++) {
									if(Toolbox.exists(file(i,"mrk"))) {
										MarkerFile mf = new MarkerFile(file(i,"mrk"));
										mf.load(); mf.del(store); mf.store();
									}
								}
								Console.println("Markers "+store+" from "+index+" to "+val+" deleted.");
							}
							else Console.println("Marker "+store+" deleted.");
							mrk.del(store); mrk.store();
						} catch(Exception x) { Console.println(x.getMessage()+" is no valid index."); }
						paintImage();
					} else Console.println("Marker "+store+" not found.");
					mode=NOOP;
				}
				break;
			case IPOL:
				if(k!='\n') read+=k;
				else { interpolate(read); mode=NOOP; }
				break;
			case NOOP: switch(k) {
				case 'q':
				case 'Q': mode=QUIT; break;
				case 'g':
				case 'G': mode=GOTO; Console.println("Type index of image:"); read=""; break;
				case 'm':
				case 'M': mode=SET1; Console.println("Select new position of marker in image:"); break;
				case 'd':
				case 'D': mode=DEL1; Console.println("Type name of marker to be deleted:"); read=""; break;
				case 'i':
				case 'I': mode=IPOL; Console.println("Type name of marker to be interpolated:"); read=""; break;
				case 'a':
				case 'A': mode=AUTO; Console.println("Autodetection started:"); break;
				case 'c':
				case 'C': checkMarkers(); break;
				case '+': index++; paintImage(); break;
				case '-': index--; paintImage(); break;
				} break;
		}
	}
	// maus-ereignisse behandeln
	public void mouseClicked(MouseEvent e) { mouse(e.getX(),e.getY(),true); }
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseMoved(MouseEvent e) { mouse(e.getX(),e.getY(),false); }
	public void mouseDragged(MouseEvent e) {}
	private void mouse(int x, int y, boolean b) {
		switch(mode) {
			case SET1: if(isInPlayer(x,y) && b) {
							Console.println("Select position of marker in detail:"); mode=SET2; }
			case NOOP:
			case GOTO: if(isInPlayer(x,y)) paintDetail(x,y); break;
			case SET2: if(isInDetail(x,y) && b) { mx+=(x-dx)/8; my+=(y-dy)/8;
							Console.println("Type name of marker:"); mode=SET3; read=""; } break;
		}
	}
	// is position x,y in player-window?
	private boolean isInPlayer(int x, int y) { return x>=px && x<(px+pw) && y>=py && y<(py+ph); }
	// is position x,y in detail-window?
	private boolean isInDetail(int x, int y) { return x>=dx && x<(dx+dw) && y>=dy && y<(dy+dh); }
}